import { Directive, ElementRef, HostListener, Input, Renderer2, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AutoSelectDirective {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
    }
    /**
     * @return {?}
     */
    onclick() {
        this.selectAll();
    }
    /**
     * @private
     * @return {?}
     */
    selectAll() {
        this.el.nativeElement.select();
    }
}
AutoSelectDirective.decorators = [
    { type: Directive, args: [{
                selector: '[powAutoSelect]'
            },] }
];
/** @nocollapse */
AutoSelectDirective.ctorParameters = () => [
    { type: ElementRef }
];
AutoSelectDirective.propDecorators = {
    onclick: [{ type: HostListener, args: ['click', ['$event.target'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AutoWidthDirective {
    /**
     * @param {?} el
     * @param {?} renderer
     */
    constructor(el, renderer) {
        this.renderer = renderer;
        // Width (in px) to add to the input's width content
        this.powExtraWidth = 0;
        // Maximal input's width
        this.powMaxWidth = 0;
        // Minimal input's width
        this.powMinWidth = 0;
        this.input = (/** @type {?} */ (el.nativeElement));
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        this.buildTwin();
        this.adjustWidth();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        document.body.removeChild(this.twin);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.setMaxWidth();
        this.setMinWidth();
    }
    /**
     * @return {?}
     */
    oninput() {
        this.adjustWidth();
    }
    /**
     * Build a twin component of the input with the same style.
     * @private
     * @return {?}
     */
    buildTwin() {
        if (this.twin) {
            document.body.removeChild(this.twin);
        }
        /** @type {?} */
        const style = window.getComputedStyle(this.input);
        this.twin = document.createElement('span');
        // default twin style
        this.renderer.setStyle(this.twin, 'position', 'absolute');
        this.renderer.setStyle(this.twin, 'top', '-999px');
        this.renderer.setStyle(this.twin, 'left', '-999px');
        this.renderer.setStyle(this.twin, 'opacity', '0');
        this.renderer.setStyle(this.twin, 'width', 'auto');
        this.renderer.setStyle(this.twin, 'white-space', 'pre');
        // twin style based on this.input
        this.renderer.setStyle(this.twin, 'border', style.border);
        this.renderer.setStyle(this.twin, 'padding', style.padding);
        this.renderer.setStyle(this.twin, 'font-family', style.fontFamily);
        this.renderer.setStyle(this.twin, 'font-feature-settings', style.fontFeatureSettings);
        this.renderer.setStyle(this.twin, 'font-size', style.fontSize);
        this.renderer.setStyle(this.twin, 'font-size-adjust', style.fontSizeAdjust);
        this.renderer.setStyle(this.twin, 'font-stretch', style.fontStretch);
        this.renderer.setStyle(this.twin, 'font-style', style.fontStyle);
        this.renderer.setStyle(this.twin, 'font-variant', style.fontVariant);
        this.renderer.setStyle(this.twin, 'font-weight', style.fontWeight);
        this.renderer.setStyle(this.twin, 'letter-spacing', style.letterSpacing);
        this.renderer.setStyle(this.twin, 'text-transform', style.textTransform);
        document.body.appendChild(this.twin);
    }
    /**
     * Adjusts the width of the input.
     * @private
     * @return {?}
     */
    adjustWidth() {
        this.clearTwinChildren();
        /** @type {?} */
        const value = this.getValue();
        this.twin.append(document.createTextNode(value));
        /** @type {?} */
        let newWidth = this.twin.getBoundingClientRect().width;
        if (this.powExtraWidth > 0) {
            newWidth += this.powExtraWidth;
        }
        this.setWidth(newWidth);
    }
    /**
     * Removes all children of twin element.
     * @private
     * @return {?}
     */
    clearTwinChildren() {
        while (this.twin.firstChild) {
            this.twin.removeChild(this.twin.firstChild);
        }
    }
    /**
     * Gets the value of the input. If it is empty, gets the value of the input's placeholder.
     * @return {?} the value of the input or of the input's placeholder.
     */
    getValue() {
        /** @type {?} */
        let value = this.input.value;
        if (!value) {
            value = this.input.placeholder || '';
        }
        return value;
    }
    /**
     * Sets the max-width of the input based on powMaxWidth.
     * @private
     * @return {?}
     */
    setMaxWidth() {
        if (this.powMaxWidth > 0) {
            this.renderer.setStyle(this.input, 'max-width', `${this.powMaxWidth}px`);
        }
    }
    /**
     * Sets the min-width of the input based on powMinWidth.
     * @private
     * @return {?}
     */
    setMinWidth() {
        if (this.powMinWidth > 0) {
            this.renderer.setStyle(this.input, 'min-width', `${this.powMinWidth}px`);
        }
    }
    /**
     * Sets the width of the input if it changed.
     * @private
     * @param {?} width the width to set.
     * @return {?}
     */
    setWidth(width) {
        if (width !== this.input.getBoundingClientRect().width) {
            this.renderer.setStyle(this.input, 'width', `${width}px`);
        }
    }
}
AutoWidthDirective.decorators = [
    { type: Directive, args: [{
                selector: '[powAutoWidth]'
            },] }
];
/** @nocollapse */
AutoWidthDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
AutoWidthDirective.propDecorators = {
    powExtraWidth: [{ type: Input }],
    powMaxWidth: [{ type: Input }],
    powMinWidth: [{ type: Input }],
    oninput: [{ type: HostListener, args: ['input', ['$event.target'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PowerfieldModule {
}
PowerfieldModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    AutoSelectDirective,
                    AutoWidthDirective
                ],
                imports: [],
                exports: [
                    AutoSelectDirective,
                    AutoWidthDirective
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AutoSelectDirective, AutoWidthDirective, PowerfieldModule };

//# sourceMappingURL=powerfield.js.map